--[[
	Project: LibNotify
	Version: 1.1.0
	Author: NerfedWar (http://www.nerfedwar.net - nerfed.war@gmail.com)
	
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
	 
]] 

-- Globals
LibNotify = {}
LibNotify.BLACK = 'd'
LibNotify.RED = 'r'
LibNotify.GREEN = 'g'
LibNotify.BLUE = 'b'

-- create context
local context = UI.CreateContext("LibNotifyContext")

-- create help stuff
local helpWindow = UI.CreateFrame("RiftWindow", "LibNotifyHelpWindow", context) 
local helpText = UI.CreateFrame("Text", "LibNotifyHelpText", helpWindow)
local helpCloseButton = UI.CreateFrame("RiftButton", "LibNotifyhelpCloseButton", helpWindow)
local helpIcon = UI.CreateFrame("Texture", "LibNotifyIcon", helpWindow)

-- set size of notification
local nHeight = 102
local nWidth = 217
local textMargin  = 16
local frameMaxLife = 100 -- the higher this value the longer notifications frames stay alive

-- tables to store the pool of frame
local notificationFramePool = {}
local notifications = {}

--[[
	Addon initialisation
]]
local function init()
	-- no init required
end


--[[
	Configures the help window
]]
local function initHelp()
	helpWindow:SetVisible(false)
	helpWindow:SetWidth(640)
	helpWindow:SetHeight(480)
	helpWindow:SetTitle("LibNotify Help")
	helpWindow:SetPoint("CENTER", UIParent, "CENTER")

	helpText:SetWidth(600)
	helpText:SetHeight(480)
	helpText:SetPoint("TOPLEFT", helpWindow, "TOPLEFT", 20, 60)
	helpText:SetText([[
	LibNotify provides an API to display/manage a stack of fading growl-like alerts and notifications.
		
	Command-line Usage: 
	
		/notify
			to display this help.
	
		/notify [colour] [message]
		/alert [colour] [message]
			[color]:
				[d]: default.
				[r]: red.
				[g]: green.
				[b]: blue.
			[message]: the message to display.
		
		e.g. /alert r Hello RIFT!
	
	API Usage: 
	
		LibNotify.notify(colour, message)
		LibNotify.alert(colour, message)
		
		e.g. /script LibNotify.alert(LibNotify.GREEN, 'Hello again RIFT!')
		
		where colour can equal: LibNotify.BLACK, LibNotify.RED, LibNotify.GREEN, LibNotify.BLUE,
		and message is a string. See README.TXT that came with this addon for more information.
							
		NerfedWar
		http://www.nerfedwar.net
		nerfed.war@gmail.com 
	    
	]])
	
	-- create the icon
	helpIcon:SetWidth(64)
	helpIcon:SetHeight(64)
	helpIcon:SetPoint("BOTTOMLEFT", helpWindow, "BOTTOMLEFT", 12, -13)
	helpIcon:SetTexture("LibNotify", "small_fangs.png")
	
	-- create the close button and add an event handler to hide the help window when it is presed
	helpCloseButton:SetText("Close")
	helpCloseButton:SetPoint("BOTTOMCENTER", helpWindow, "BOTTOMCENTER", 0, -20)
	function helpCloseButton.Event:LeftPress()
		helpWindow:SetVisible(false)
	end		

end


--
local function Split(str, delim, maxNb)
	-- Eliminate bad cases...
	if string.find(str, delim) == nil then
		return { str }
	end
	if maxNb == nil or maxNb < 1 then
		maxNb = 0    -- No limit
	end
	local result = {}
	local pat = "(.-)" .. delim .. "()"
	local nb = 0
	local lastPos
	for part, pos in string.gfind(str, pat) do
		nb = nb + 1
		result[nb] = part
		lastPos = pos
		if nb == maxNb then break end
	end
	-- Handle the last field
	result[nb + 1] = string.sub(str, lastPos)
	return result
end


-- creates a pool of 10 blank notification frames and place them in the appropriate places
local function createNotificationFrames()

	for i=1,10 do

		-- create new notification container box at topright
		local nBox = UI.CreateFrame("Frame", "LibNotifyNotificationFrame", context)
		nBox:SetWidth(nWidth)
		nBox:SetHeight(nHeight)
		nBox:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", 0, (i-1)*nHeight)
		nBox:SetVisible(false)
		nBox:SetLayer(1)
		-- add on click event
		function nBox.Event:LeftClick()
			LibNotify.closeNotification(i)
		end			

		-- create notification text
		local nText = UI.CreateFrame("Text", "LibNotifyNotificationText", nBox)
		nText:SetPoint("TOPLEFT", nBox, "TOPLEFT", textMargin, textMargin)
		nText:SetWidth(nWidth-(2*textMargin))
		nText:SetHeight(nHeight-(2*textMargin))			
		nText:SetFontColor(1,1,1)
		nText:SetFontSize(14)
		nText:SetWordwrap(true)
		nText:SetText(' ')	
		nText:SetAlpha(1)	
		nText:SetLayer(2)		
		
		-- create notification background
		local nBack = UI.CreateFrame("Texture", "LibNotifyNotificationBack", nBox)
		nBack:SetWidth(nWidth)
		nBack:SetHeight(nHeight)
		nBack:SetPoint("TOPRIGHT", nBox, "TOPRIGHT")
		nBack:SetTexture("LibNotify", "images/notification_black.png")
		nBack:SetAlpha(1)	
		nBack:SetLayer(1)

		-- create new notification glow
		local nGlow = UI.CreateFrame("Texture", "LibNotifyNotificationGlow", nBox)
		nGlow:SetWidth(nWidth)
		nGlow:SetHeight(nHeight)
		nGlow:SetPoint("TOPRIGHT", nBox, "TOPRIGHT")
		nGlow:SetTexture("LibNotify", "images/glow.png")
		nGlow:SetAlpha(0)			
		nGlow:SetLayer(1)
		
		-- add to notifications table
		notificationFramePool[i] = {nBox, nBack, nText, nGlow}
		
	end
end

function LibNotify.closeNotification(num)
	notifications[num][4]=0
end

-- 
local function addNotification(nType, param)

	-- split param by words to get the parameters
	local params=Split(param, " ", 1)
	
	-- validate type parameter 
	if ( (nType~='n') and (nType~='a') ) then
		print("Invalid LibNotify type.")
		helpWindow:SetVisible(true)
		return
	end
	
	-- validate colour parameter
	if ( (params[1]~='d') and (params[1]~='r') and (params[1]~='g') and (params[1]~='b') ) then
		print("Invalid LibNotify colour.")
		helpWindow:SetVisible(true)
		return
	end
	
	-- validate message
	if(not params[2]) then
		print("Invalid LibNotify message.")
		helpWindow:SetVisible(true)
		return
	end

	-- add to notifications table (type, colour, message, life)
	table.insert(notifications, {nType, params[1], params[2], frameMaxLife})
	
end


-- reverse the order of a tables elements
function table.reverse ( tab )
	local size = #tab
	local newTable = {}
	for i,v in ipairs ( tab ) do
		newTable[size-i] = v
	end
	return newTable
end


-- called by the notify slash command
local function handleCommand(nType, param)
	if( (param ~= nil) and (param ~= "")) then
		if (param == "help" or (param == "h")) then
			helpWindow:SetVisible(true)
		else
			addNotification(nType, param)
		end
	else
		helpWindow:SetVisible(true)  
	end
end


-- called many times a second by the Event.System.Update.Begin event
local lastCall = Inspect.Time.Real()
local function Update()
	
	-- we only want this running once every 10th of a second or we'll
	-- break someones CPU
	local now = Inspect.Time.Real()
	if (now<lastCall+0.1) then 
		return
	end
	lastCall = now
	
	-- sort notifications by frameLife
	local orderedNotifications = table.sort(
			notifications, function(a,b) 
				return a[4]>b[4] 
			end
		)
	
	-- loop through all notifications
	local frameId = 1
	for i,v in ipairs ( notifications ) do
	
		if(frameId>10) then
			break
		end

		local box = notificationFramePool[frameId][1]
		local back = notificationFramePool[frameId][2]
		local text = notificationFramePool[frameId][3]
		local glow = notificationFramePool[frameId][4]
		
		box:SetVisible(true)
		back:SetAlpha(v[4]/frameMaxLife)
		text:SetAlpha((v[4]*4)/frameMaxLife)

		if(v[1]=='a') then
			local _,rem = math.modf(math.abs(((v[4]/10)-0.5)*2))
			glow:SetAlpha(rem*(v[4]/frameMaxLife))
		end
		if(v[2]=='d') then back:SetTexture("LibNotify", "images/notification_black.png") end
		if(v[2]=='r') then back:SetTexture("LibNotify", "images/notification_red.png") end
		if(v[2]=='b') then back:SetTexture("LibNotify", "images/notification_blue.png")  end
		if(v[2]=='g') then back:SetTexture("LibNotify", "images/notification_green.png") end
		text:SetText(v[3])
		v[4]=v[4]-1
		if (v[4]<=0) then
			box:SetVisible(false)
			back:SetAlpha(1)
			text:SetAlpha(1)
			text:SetText(' ')
			glow:SetAlpha(0)
			table.remove(notifications, i)
		end
		frameId=frameId+1
		
	end
	
end

-- API FUNCTIONS --
function LibNotify.notify(colour, message)
	addNotification('n', colour..' '..message)
end

function LibNotify.alert(colour, message) 
	addNotification('a', colour..' '..message)
end


-- Register the slash commands
table.insert(Command.Slash.Register("notify"), {function (params) handleCommand('n', params) end, "LibNotify", "Slash command"})
table.insert(Command.Slash.Register("alert"), {function (params) handleCommand('a',params) end, "LibNotify", "Slash command"})
table.insert(Event.System.Update.Begin, {Update, "LibNotify", "Update"})

-- initialisation
init()
initHelp()
createNotificationFrames()